﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Infragistics.Controls.Charts;
using System.Collections;
using System.Diagnostics;

namespace ScatterAndColumn
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            DataContext = Data.GetData(10, 10, 100);
        }
    }

    public class IntToColorConverter : IMultiValueConverter
    {

        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if ((int)values[0] < (int)values[1])
            {
                return new SolidColorBrush(Colors.Green);
            }
            else
            {
                return new SolidColorBrush(Colors.Red);
            }
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }


    public class Data : INotifyPropertyChanged
    {
        private int open;
        private int close;
        private int high;
        private int low;
        private int value;
        private string label;

        private int threshold;

        public int Threshold 
        {
            get { return threshold; }
            set { threshold = value; PropChanged("Threshold"); }
        }


        public string Label
        {
            get { return label; }
            set
            {
                label = value;
                PropChanged("Label");
            }
        }

        public int Open
        {
            get { return open; }
            set
            {
                open = value;
                PropChanged("Open");
            }
        }

        public int Close
        {
            get { return close; }
            set
            {
                close = value;
                PropChanged("Close");
            }
        }

        public int High
        {
            get { return high; }
            set
            {
                high = value;
                PropChanged("High");
            }
        }

        public int Low
        {
            get { return low; }
            set
            {
                low = value;
                PropChanged("Low");
            }
        }

        public int Value
        {
            get { return value; }
            set
            {
                this.value = value;
                PropChanged("Value");
            }
        }


        public static ObservableCollection<Data> GetData(int points, int rangeStart, int rangeEnd)
        {
            ObservableCollection<Data> data = new ObservableCollection<Data>();
            Random r = new Random();
            for (int i = 0; i < points; i++)
            {
                data.Add(new Data
                {
                    Label = "Item " + i,
                    Open = r.Next(rangeStart, rangeEnd),
                    Close = r.Next(rangeStart, rangeEnd),
                    High = r.Next(rangeStart, rangeEnd),
                    Low = r.Next(rangeStart, rangeEnd),
                    Value = r.Next(rangeStart, rangeEnd),
                    Threshold = r.Next(rangeStart, rangeEnd),
                });
            }

            return data;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public void PropChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }

    public static class StyleBinder
    {
        public static readonly DependencyProperty StyleBinderHelperProperty =
            DependencyProperty.RegisterAttached(
            "StyleBinderHelper",
            typeof(StyleBinderHelper),
            typeof(StyleBinder),
            new PropertyMetadata(null,
                (o, e) => StyleBinderHelperChanged(o, e)));

        private static void StyleBinderHelperChanged(
            DependencyObject o,
            DependencyPropertyChangedEventArgs e)
        {
            FrameworkElement ele = o as FrameworkElement;
            if (ele == null)
            {
                return;
            }

            BindingExpression be =
                ele.GetBindingExpression(StyleBinder.StyleBinderContextProperty);

            if (be == null)
            {
                ele.SetBinding(StyleBinderContextProperty,
                    new Binding());
            }

            if (e.NewValue != null)
            {
                ((StyleBinderHelper)e.NewValue).PushContent(ele);
            }
        }

        public static void SetStyleBinderHelper(
           DependencyObject target,
           StyleBinderHelper value)
        {
            target.SetValue(StyleBinderHelperProperty, value);
        }

        public static StyleBinderHelper GetStyleBinderHelper(
            DependencyObject target)
        {
            return (StyleBinderHelper)target.GetValue(StyleBinderHelperProperty);
        }

        public static readonly DependencyProperty StyleBinderContentProperty =
            DependencyProperty.RegisterAttached(
            "StyleBinderContent",
            typeof(FrameworkElement),
            typeof(StyleBinder),
            new PropertyMetadata(null,
                (o, e) => StyleBinderContentChanged(o, e)));

        private static void StyleBinderContentChanged(
            DependencyObject o,
            DependencyPropertyChangedEventArgs e)
        {
        }

        public static void SetStyleBinderContent(
            DependencyObject target,
            FrameworkElement value)
        {
            target.SetValue(StyleBinderContentProperty, value);
        }

        public static FrameworkElement GetStyleBinderContent(
            DependencyObject target)
        {
            return (FrameworkElement)target.GetValue(StyleBinderContentProperty);
        }


        public static readonly DependencyProperty StyleBinderContextProperty =
            DependencyProperty.RegisterAttached(
            "StyleBinderContext",
            typeof(object),
            typeof(StyleBinder),
            new PropertyMetadata(null,
                (o, e) => StyleBinderContextChanged(o, e)));

        private static void StyleBinderContextChanged(
            DependencyObject o,
            DependencyPropertyChangedEventArgs e)
        {
            FrameworkElement content =
                StyleBinder.GetStyleBinderContent(o);
            if (content != null)
            {
                content.DataContext = new HelperContext()
                {
                    OuterContext = e.NewValue,
                    Owner = o as FrameworkElement
                };
            }
        }

        public static void SetStyleBinderContext(
            DependencyObject target,
            object value)
        {
            target.SetValue(StyleBinderContextProperty, value);
        }

        public static object GetStyleBinderContext(
            DependencyObject target)
        {
            return target.GetValue(StyleBinderContextProperty);
        }

    }

    public class StyleBinderHelper
        : FrameworkElement
    {
        internal void PushContent(FrameworkElement ele)
        {
            if (Template == null)
            {
                return;
            }
            FrameworkElement content = Template.LoadContent() as FrameworkElement;
            if (content == null)
            {
                return;
            }
            content.DataContext = new HelperContext()
            {
                OuterContext = StyleBinder.GetStyleBinderContext(ele),
                Owner = ele
            };

            StyleBinder.SetStyleBinderContent(ele, content);
        }

        public DataTemplate Template { get; set; }
    }

    public class HelperContext
        : INotifyPropertyChanged
    {
        private object _outerContext;
        public object OuterContext
        {
            get { return _outerContext; }
            set
            {
                _outerContext = value;
                RaisePropertyChanged("OuterContext");
            }
        }

        private FrameworkElement _owner;
        public FrameworkElement Owner
        {
            get { return _owner; }
            set
            {
                _owner = value;
                RaisePropertyChanged("Owner");
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        private void RaisePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this,
                    new PropertyChangedEventArgs(propertyName));
            }
        }
    }

    public class RectangleColorChanger
        : FrameworkElement
    {
        public static readonly DependencyProperty RectangleProperty =
            DependencyProperty.Register(
            "Rectangle",
            typeof(Rectangle),
            typeof(RectangleColorChanger),
            new PropertyMetadata(null,
                (o, e) => (o as RectangleColorChanger)
                .RectangleChanged(e)));

        private double _oldMiddle = double.NaN;

        private void UpdateColor()
        {
            if (XAxis == null)
            {
                return;
            }
            if (Rectangle == null)
            {
                return;
            }

            Rect viewport = new Rect(0, 0, XAxis.ActualWidth, XAxis.ActualHeight);
            Rect window = XAxis.Chart.WindowRect;
            double middle = Rectangle.TransformToVisual(XAxis).Transform(new Point(Rectangle.ActualWidth / 2.0, 0)).X;
            _oldMiddle = middle;
            middle = window.Left + window.Width * (middle - viewport.Left) / viewport.Width;
            double left = XAxis.GetUnscaledValue(viewport.Left, window, viewport);
            double right = XAxis.GetUnscaledValue(viewport.Right, window, viewport);
            double windowX = (middle - window.Left) / window.Width;
            double index = left + (windowX * (right - left));
            index -= .5;

            int indexNumber = (int)Math.Round(index);

            MultiBinding binding = new MultiBinding();
            binding.Converter = new IntToColorConverter();

            binding.Bindings.Add(new Binding(ColorPath)
                {
                    Source = XAxis.ItemsSource
                    .OfType<object>()
                    .Skip(indexNumber)
                    .Take(1)
                    .FirstOrDefault()
                });

            binding.Bindings.Add(new Binding(((Rectangle.DataContext as DataContext).Series as ColumnSeries).ValueMemberPath)
            {
                Source = XAxis.ItemsSource
                .OfType<object>()
                .Skip(indexNumber)
                .Take(1)
                .FirstOrDefault()
            });

            SetBinding(ColorProperty, binding);
                
        }

        private void UpdateRectangleColor()
        {
            if (Rectangle == null)
            {
                return;
            }
            Rectangle.Fill = Color;
            Rectangle.LayoutUpdated += new EventHandler(Rectangle_LayoutUpdated);
        }

        void Rectangle_LayoutUpdated(object sender, EventArgs e)
        {
            if (Rectangle.Parent == null)
            {
                Rectangle.LayoutUpdated -= Rectangle_LayoutUpdated;
                return;
            }
            double middle = Rectangle.TransformToVisual(XAxis).Transform(new Point(Rectangle.ActualWidth / 2.0, 0)).X;
            if (middle != _oldMiddle)
            {
                UpdateColor();
            }
        }

        private void RectangleChanged(DependencyPropertyChangedEventArgs e)
        {
            UpdateColor();
        }

        public Rectangle Rectangle
        {
            get { return (Rectangle)GetValue(RectangleProperty); }
            set { SetValue(RectangleProperty, value); }
        }

        public static readonly DependencyProperty XAxisProperty =
           DependencyProperty.Register(
           "XAxis",
           typeof(CategoryXAxis),
           typeof(RectangleColorChanger),
           new PropertyMetadata(null, (o, e) => (o as RectangleColorChanger)
               .XAxisChanged(e)));

        private void XAxisChanged(
            DependencyPropertyChangedEventArgs e)
        {
            UpdateColor();
        }

        public CategoryXAxis XAxis
        {
            get { return (CategoryXAxis)GetValue(XAxisProperty); }
            set { SetValue(XAxisProperty, value); }
        }

        public static readonly DependencyProperty ColorProperty =
          DependencyProperty.Register(
          "Color",
          typeof(Brush),
          typeof(RectangleColorChanger),
          new PropertyMetadata(null, (o, e) => (o as RectangleColorChanger)
              .ColorChanged(e)));

        private void ColorChanged(
            DependencyPropertyChangedEventArgs e)
        {
            UpdateRectangleColor();
        }

        public Brush Color
        {
            get { return (Brush)GetValue(ColorProperty); }
            set { SetValue(ColorProperty, value); }
        }

        public static readonly DependencyProperty ColorPathProperty =
          DependencyProperty.Register(
          "ColorPath",
          typeof(string),
          typeof(RectangleColorChanger),
          new PropertyMetadata("", (o, e) => (o as RectangleColorChanger)
              .ColorPathChanged(e)));

        private void ColorPathChanged(
            DependencyPropertyChangedEventArgs e)
        {
            UpdateColor();
        }

        public string ColorPath
        {
            get { return (string)GetValue(ColorPathProperty); }
            set { SetValue(ColorPathProperty, value); }
        }

    }  


//    public static class StyleBinder
//    {
//        public static readonly DependencyProperty StyleBinderHelperProperty =
//            DependencyProperty.RegisterAttached(
//            "StyleBinderHelper",
//            typeof(StyleBinderHelper),
//            typeof(StyleBinder),
//            new PropertyMetadata(null,
//                (o, e) => StyleBinderHelperChanged(o, e)));

//        private static void StyleBinderHelperChanged(
//            DependencyObject o,
//            DependencyPropertyChangedEventArgs e)
//        {
//            FrameworkElement ele = o as FrameworkElement;
//            if (ele == null)
//            {
//                return;
//            }

//            BindingExpression be =
//                ele.GetBindingExpression

//(
//                StyleBinder.StyleBinderContextProperty);

//            if (be == null)
//            {
//                ele.SetBinding(StyleBinderContextProperty,
//                    new Binding());
//            }

//            if (e.NewValue != null)
//            {
//                ((StyleBinderHelper)e.NewValue).PushContent(ele);
//            }
//        }

//        public static void SetStyleBinderHelper(
//           DependencyObject target,
//           StyleBinderHelper value)
//        {
//            target.SetValue(StyleBinderHelperProperty, value);
//        }

//        public static StyleBinderHelper GetStyleBinderHelper(
//            DependencyObject target)
//        {
//            return (StyleBinderHelper)target.GetValue(StyleBinderHelperProperty);
//        }

//        public static readonly DependencyProperty StyleBinderContentProperty =
//            DependencyProperty.RegisterAttached(
//            "StyleBinderContent",
//            typeof(FrameworkElement),
//            typeof(StyleBinder),
//            new PropertyMetadata(null,
//                (o, e) => StyleBinderContentChanged(o, e)));

//        private static void StyleBinderContentChanged(
//            DependencyObject o,
//            DependencyPropertyChangedEventArgs e)
//        {
//        }

//        public static void SetStyleBinderContent(
//            DependencyObject target,
//            FrameworkElement value)
//        {
//            target.SetValue(StyleBinderContentProperty, value);
//        }

//        public static FrameworkElement GetStyleBinderContent(
//            DependencyObject target)
//        {
//            return (FrameworkElement)target.GetValue(StyleBinderContentProperty);
//        }


//        public static readonly DependencyProperty StyleBinderContextProperty =
//            DependencyProperty.RegisterAttached(
//            "StyleBinderContext",
//            typeof(object),
//            typeof(StyleBinder),
//            new PropertyMetadata(null,
//                (o, e) => StyleBinderContextChanged(o, e)));

//        private static void StyleBinderContextChanged(
//            DependencyObject o,
//            DependencyPropertyChangedEventArgs e)
//        {
//            FrameworkElement content =
//                StyleBinder.GetStyleBinderContent(o);
//            if (content != null)
//            {
//                content.DataContext = new HelperContext()
//                {
//                    OuterContext = e.NewValue,
//                    Owner = o as FrameworkElement
//                };
//            }
//        }

//        public static void SetStyleBinderContext(
//            DependencyObject target,
//            object value)
//        {
//            target.SetValue(StyleBinderContextProperty, value);
//        }

//        public static object GetStyleBinderContext(
//            DependencyObject target)
//        {
//            return target.GetValue(StyleBinderContextProperty);
//        }

//    }

//    public class StyleBinderHelper
//        : FrameworkElement
//    {
//        internal void PushContent(FrameworkElement ele)
//        {
//            if (Template == null)
//            {
//                return;
//            }
//            FrameworkElement content = Template.LoadContent() as FrameworkElement;
//            if (content == null)
//            {
//                return;
//            }

            
//            HelperContext hc = new HelperContext()
//            {
            
//                OuterContext = StyleBinder.GetStyleBinderContext(ele),
//                Owner = ele
//            };

//            if (ele.DataContext is DataContext && ele.Parent is Canvas && (ele.Parent as Canvas).Name.Equals("RootCanvas"))
//            {
//                Debug.WriteLine((ele.Parent as Canvas).Children.Contains(ele));
//                if((ele.Parent as Canvas).Children.IndexOf(ele) >-1)
//                    hc.DataItem = ((ele.DataContext as DataContext).Series.DataContext as IEnumerable<Data>).ElementAt((ele.Parent as Canvas).Children.IndexOf(ele));
//            }

//            content.DataContext = hc;


//            StyleBinder.SetStyleBinderContent(ele, content);
//        }

//        public DataTemplate Template { get; set; }
//    }

//    public class HelperContext
//        : INotifyPropertyChanged
//    {
//        private object _outerContext;
//        public object OuterContext
//        {
//            get { return _outerContext; }
//            set
//            {
//                _outerContext = value;
//                RaisePropertyChanged("OuterContext");
//            }
//        }

//        private object dataItem;

//        public object DataItem
//        {
//            get { return dataItem; }
//            set { dataItem = value; RaisePropertyChanged("DataItem"); }
//        }


//        private FrameworkElement _owner;
//        public FrameworkElement Owner
//        {
//            get { return _owner; }
//            set
//            {
//                _owner = value;
//                RaisePropertyChanged("Owner");
//            }
//        }

//        public event PropertyChangedEventHandler PropertyChanged;

//        private void RaisePropertyChanged(string propertyName)
//        {
//            if (PropertyChanged != null)
//            {
//                PropertyChanged(this,
//                    new PropertyChangedEventArgs(propertyName));
//            }
//        }
//    }

//    public class RectangleColorChanger
//        : FrameworkElement
//    {
//        public static readonly DependencyProperty ValueProperty =
//            DependencyProperty.Register(
//            "Value",
//            typeof(double),
//            typeof(RectangleColorChanger),
//            new PropertyMetadata(0.0, (o, e) => (o as RectangleColorChanger)
//                .ValueChanged(e)));

//        public double Value
//        {
//            get { return (double)GetValue(ValueProperty); }
//            set { SetValue(ValueProperty, value); }
//        }

//        public static readonly DependencyProperty ThresholdProperty =
//            DependencyProperty.Register(
//            "Threshold",
//            typeof(double),
//            typeof(RectangleColorChanger),
//            new PropertyMetadata(0.0, (o, e) => (o as RectangleColorChanger)
//                .ThresholdChanged(e)));

//        private void ThresholdChanged(
//            DependencyPropertyChangedEventArgs e)
//        {
//            UpdateColor();
//        }

//        public double Threshold
//        {
//            get { return (double)GetValue(ThresholdProperty); }
//            set { SetValue(ThresholdProperty, value); }
//        }

//        public static readonly DependencyProperty RectangleProperty =
//            DependencyProperty.Register(
//            "Rectangle",
//            typeof(Rectangle),
//            typeof(RectangleColorChanger),
//            new PropertyMetadata(null,
//                (o, e) => (o as RectangleColorChanger)
//                .RectangleChanged(e)));

//        private void UpdateColor()
//        {
//            if (YAxis == null)
//            {
//                return;
//            }
//            if (Rectangle == null)
//            {
//                return;
//            }

//            Rect viewport = new Rect(0, 0, YAxis.ActualWidth, YAxis.ActualHeight);
//            double zero = YAxis.GetScaledValue(YAxis.ReferenceValue, YAxis.Chart.WindowRect, viewport);
//            double top = Rectangle.TransformToVisual(YAxis).Transform(new Point(0, 0)).Y;
//            double bottom = Rectangle.TransformToVisual(YAxis).Transform(new Point(0, Rectangle.Height)).Y;
//            double val = 0;

//            Rect window = XAxis.Chart.WindowRect;
//            double middle = Rectangle.TransformToVisual(XAxis).Transform(new Point(Rectangle.ActualWidth / 2.0, 0)).X;
//            _oldMiddle = middle;
//            middle = window.Left + window.Width * (middle - viewport.Left) / viewport.Width;
//            double left = XAxis.GetUnscaledValue(viewport.Left, window, viewport);
//            double right = XAxis.GetUnscaledValue(viewport.Right, window, viewport);
//            double windowX = (middle - window.Left) / window.Width;
//            double index = left + (windowX * (right - left));
//            index -= .5;

//            int indexNumber = (int)Math.Round(index);   
//            if (top < zero && bottom <= zero)
//            {
//                val = YAxis.GetUnscaledValue(top, YAxis.Chart.WindowRect, viewport);
//            }
//            else
//            {
//                val = YAxis.GetUnscaledValue(bottom, YAxis.Chart.WindowRect, viewport);
//            }

//            if (val < Threshold)
//            {
//                Rectangle.Fill = new SolidColorBrush(Colors.Red);
//            }
//            else
//            {
//                Rectangle.Fill = new SolidColorBrush(Colors.Green);
//            }
//        }

//        private void RectangleChanged(DependencyPropertyChangedEventArgs e)
//        {
//            UpdateColor();
//        }

//        public Rectangle Rectangle
//        {
//            get { return (Rectangle)GetValue(RectangleProperty); }
//            set { SetValue(RectangleProperty, value); }
//        }

//        private void ValueChanged(DependencyPropertyChangedEventArgs e)
//        {
//            UpdateColor();
//        }

//        public static readonly DependencyProperty YAxisProperty =
//           DependencyProperty.Register(
//           "YAxis",
//           typeof(NumericYAxis),
//           typeof(RectangleColorChanger),
//           new PropertyMetadata(null, (o, e) => (o as RectangleColorChanger)
//               .YAxisChanged(e)));

//        private void YAxisChanged(
//            DependencyPropertyChangedEventArgs e)
//        {
//            UpdateColor();
//        }

//        public NumericYAxis YAxis
//        {
//            get { return (NumericYAxis)GetValue(YAxisProperty); }
//            set { SetValue(YAxisProperty, value); }
//        }

//    }
}
